/***********************************************************************/
/*                                                                     */
/*   InstantUnicode :: Insert Unicode character by typing code point   */
/*                                                                     */
/*   [Ver: 1.01]      [Author: M. Autret]         [Modif: 20/12/03]    */
/*   [Lang: EN]       [Req: InDesign CS6-CC]      [Creat: 20/11/03]    */
/*   [Type: FREE]                                                      */
/*                                                                     */
/*                                                                     */
/*      Installation:                                                  */
/*                                                                     */
/*      1) Place the current file into the Scripts/Scripts Panel/      */
/*         folder                                                      */
/*                                                                     */
/*      2) Start InDesign                                              */
/*                                                                     */
/*      3) Exec the script from Window>Utilities>Scripts               */
/*         + double-click on the script file name                      */
/*                                                                     */
/*      Bugs & Feedback : marc{at}indiscripts{dot}com                  */
/*                        www.indiscripts.com                          */
/*                                                                     */
/***********************************************************************/

#targetengine 'InstantUnicode 1.01'

'function' == typeof($.global['Idle'+$.engineName]) ||
($.global['Idle'+$.engineName] = function IdleCallback(  x,i,s,p,t,r)
//----------------------------------------------------------
{
	const RE_uHHHH =   /u[0-9a-fA-F]{4}$/;
	const RE_UHHHHH =  /U[12eEfF][0-9a-fA-F]{4}$/;
	const RE_LAST_IP = /\/insertion-point\[\d+\]$/;

	// Preconditions.
	// ---
	if( !app.properties.activeDocument ) return;
	if( !(x=app.properties.selection) ) return;
	if( 1 != x.length ) return;
	if( !((x=x[0]) instanceof InsertionPoint) ) return;
	if( 5 > (i=x.index) ) return;
	
	var m = app.menus.itemByName('$ID/Main').submenus.itemByName('$ID/Edit');
	m = m.menuItems[1].name;


	// s :: `<container_spec>/character[`
	// ---
	s = x.toSpecifier().replace(RE_LAST_IP,'/character[');
	
	// Parse text range before IP.
	// ---
	r = false;
	while( 1 )
	{
		// `uHHHH ` form?
		// ---
		p = i-5;
		t = '(' + s + p + '] to ' + s + (4+p) + '])';
		if( !(x=resolve(t)).isValid ) return;
		if( RE_uHHHH.test(t=x.contents) )
		{
			t = parseInt(t.slice(1),16); // 0x0000..0xFFFF
			r = String.fromCharCode(t);
			break;
		}

		// `UHHHHH ` form?
		// ---
		if( 6 > i ) return;
		t = '(' + s + (-1+p) + '] to ' + s + (4+p) + '])';
		if( !(x=resolve(t)).isValid ) return;
		if( RE_UHHHHH.test(t=x.contents) )
		{
			t = parseInt(t.slice(1),16); // 0x10000..0xF0000
			t -= 0x10000;
			
			// Surrogate pair (r.length==2)
			// ---
			r = String.fromCharCode( 0xD800+Math.floor(t/0x400),0xDC00+(t%0x400) );
			break;
		}
		return;
	}
	
	// Forbidden or dangerous code points (InDesign.)
	// ---
	// Null                                U+0000 <ctrl> NULL
	// Table Anchor                        U+0016 <ctrl> SYNCHRONOUS IDLE   /aka Table
	// Table Middle (ghost table char)     U+0017 <ctrl> EOT BLOCK          /aka TableContinued
	// Non Roman Special Glyph             U+001A <ctrl> SUBSTITUTE         /aka NonRomanSpecialGlyph
	// Index Marker                        U+FEFF ZERO WIDTH NO-BREAK SPACE /aka ZeroSpaceNoBreak
	//    Note Marker                      idem
	//    Conditional Text Marker          idem
	//    Cross-ref                        idem
	// Anchored Object Marker              U+FFFC OBJECT REPLACEMENT CHARACTER
	//    Placeholder for unspec. object   idem
	// Replacement Character               U+FFFD REPLACEMENT CHARACTER      /aka ReplacementCharacter
	// ---
	const SKIP = "\0\x16\x17\x1A\uFEFF\uFFFC\uFFFD";
	if( (!r) || 0 <= "\0\x16\x17\x1A\uFEFF\uFFFC\uFFFD".indexOf(r) ) return;

	// DOM commands.
	// ---
	x.select();
	t = x.insertionPoints[1].getElements()[0];
	app.doScript("x.contents=r; if( t.isValid )t.select();", void 0, void 0, +UndoModes.entireScript, callee.displayName||void 0);
});

'string' == typeof $.global['Idle'+$.engineName].help || ($.global['Idle'+$.engineName].help =
"""-----------------------

> To insert a Unicode character in your text, type 'u' followed by the 4-digit codepoint (e.g. 'u204B').

> If your font supports codepoints greater than `FFFF`, type 'U' followed by the 5-digit codepoint (e.g. 'U1F7AB').

> Re-run this script to turn it off.

[Tip] Put the JSX file in your `startup scripts` folder to make it automatically active.

-----------------------"""
);

(function(/*IdleTasks*/tasks,/*str*/name,/*uint>10*/rate,/*fct*/callback,  t)
//----------------------------------------------------------
// Toggle the listener ON / OFF.
{
	callback.displayName || (callback.displayName = name.replace(/\s+\d.*$/,''));

	if( (t=tasks.itemByName(name)).isValid )
	{
		t.eventListeners.everyItem().remove();
		t.remove();
		alert(name + " is turned OFF.");
	}
	else
	{
		// Prevent alert on automatic startup.
		// ---
		t = app.performanceMetric(+PerformanceMetricOptions.MINISAVE_COUNT);
		if( 0 < t ) alert(name + " is turned ON.\r\r" + callback.help);
		tasks.add({name:name, sleep:rate}).addEventListener(IdleEvent.ON_IDLE,callback, false);
	}

})(app.idleTasks, $.engineName, 25, $.global['Idle'+$.engineName]);